#!/usr/bin/python
# -*- coding: utf-8 -*-


# http://jenyay.net/Programming/WxSizer

import wx
import wx.grid as gridlib

import hpd20

from instrumentname import get_instrument_pitch, get_instrument_name, get_complete_instrument_list

wx.SetDefaultPyEncoding('utf-8')

class MyForm(wx.Frame):
    def __init__(self, file_name):
        self.hpd = hpd20.HPD(file_name)
        wx.Frame.__init__(self, parent=None, title="HPD-20 Editor", size=(1050, 860))
        self.panel = wx.Panel(self)
        self.layout_menu()
        self.layout_kit_grid()

    def layout_menu(self):
        menuBar = wx.MenuBar()

        menu1 = wx.Menu()
        menuBar.Append(menu1, '&File')
        open_item = menu1.Append(wx.ID_OPEN, '&Open memory dump')

        self.Bind(wx.EVT_MENU, self.load_memory_backup, open_item)
        save_item = menu1.Append(wx.ID_SAVE, '&Save memory dump')
        self.Bind(wx.EVT_MENU, self.save_memory_backup, save_item)
        menu1.AppendSeparator()
        loadkit_item = menu1.Append(wx.ID_ANY, 'Load Kit (overwrites current)')
        self.Bind(wx.EVT_MENU, self.load_kit, loadkit_item)
        savekit_item = menu1.Append(wx.ID_ANY, 'Save Kit')
        self.Bind(wx.EVT_MENU, self.save_kit, savekit_item)
        menu1.AppendSeparator()
        fitem = menu1.Append(wx.ID_EXIT, 'Quit', 'Quit application')
        menu6 = wx.Menu()
        menuBar.Append(menu6, '&Tools')
        menu7 = wx.Menu()
        menuBar.Append(menu7, '&Macros')
        menu7.Append(wx.ID_ANY, 'Set Scale')
        menu7.Append(wx.ID_ANY, 'Pan')

        menu8 = wx.Menu()
        menuBar.Append(menu8, '&Help')

        self.SetMenuBar(menuBar)

    def layout_kit_grid(self):
        self.columns = ['Layer', 'fade', 'trigger', 'velo', 'mutgrp', 'mn/ply','Instrument', 'nr', 'vol', 'cents', 'note', 'muff', 'color', 'swp', 'ambsend']
        self.pad_names = ['M1 ◵', 'M2 ◶', 'M3 ◴', 'M4 ◷', 'M5 ●',
                          'S1', 'S2', 'S3', 'S4',
                          'S5', 'S6', 'S7', 'S8',
                          'D-Beam', 'Head', 'Rim', 'HH']
        self.kit_grid = gridlib.Grid(self.panel)
        self.kit_grid.CreateGrid(len(self.pad_names)*2, len(self.columns))
        for i in range(len(self.columns)):
            self.kit_grid.SetColLabelValue(i, self.columns[i])
            self.kit_grid.SetColSize(i, 48)
        for i in range(len(self.pad_names)):
            self.kit_grid.SetRowSize(i, 24)
            self.kit_grid.SetRowLabelValue(i * 2, self.pad_names[i])
            self.kit_grid.SetRowLabelValue(i * 2+1, '')
        self.kit_grid.SetColSize(6, 200)
        #        self.kit_grid.SetCellFont(0, 0, wx.Font(12, wx.ROMAN, wx.ITALIC, wx.NORMAL))
        print self.kit_grid.GetCellValue(0, 0)

        self.kit_grid.SetCellTextColour(1, 1, wx.RED)
        self.kit_grid.SetCellBackgroundColour(2, 2, wx.CYAN)

        #toolbar2 = wx.ToolBar(self, wx.TB_HORIZONTAL | wx.TB_TEXT)

        self.fill_kit(130)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((50, 50), 0)
        sizer.Add(self.kit_grid)
        sizer.Add((50, 50), 0)
        self.panel.SetSizer(sizer)


    def fill_kit(self, kit):
        instr_index = 6
        layer_type = ['off', 'mix', 'velo mix', 'velo fade', 'velo sw']

        for i in range(17):
            pad = self.hpd.pads.get_pad(kit * hpd20.HPD.PADS_PER_KIT + i)
            if pad.get_layer() == 0:
                for j in range(len(self.columns)):
                    self.kit_grid.SetCellTextColour(i * 2 + 1, j, wx.YELLOW)
            else:
                for j in range(len(self.columns)):
                    self.kit_grid.SetCellTextColour(i * 2 + 1, j, wx.BLACK)
            self.kit_grid.SetCellValue(i * 2, 0, layer_type[pad.get_layer()])
            for instr in range(2):
                choice_editor = gridlib.GridCellChoiceEditor(get_complete_instrument_list(), True)
                self.kit_grid.SetCellEditor(i * 2 + instr, instr_index+0, choice_editor)
                self.kit_grid.SetCellValue(i * 2 + instr, instr_index+0, get_instrument_name(pad.get_patch(instr)))
                self.kit_grid.SetCellValue(i * 2 + instr, instr_index+1, str(pad.get_patch(instr)+1))
                self.kit_grid.SetCellEditor(i * 2 + instr, instr_index+2, gridlib.GridCellNumberEditor(0, 1900))
                self.kit_grid.SetCellValue(i * 2 + instr, instr_index+2, str(pad.get_volume(instr)))
                self.kit_grid.SetCellEditor(i * 2 + instr, instr_index+2, gridlib.GridCellNumberEditor(0, 127))
                pitch = pad.get_pitch(instr)
                self.kit_grid.SetCellValue(i * 2 + instr, instr_index+3, str(pitch))
                self.kit_grid.SetCellEditor(i * 2 + instr, instr_index+3, gridlib.GridCellNumberEditor(-2400, 2400))
                pitch = pitch + get_instrument_pitch(pad.get_patch(instr))
                real_note = hpd20.get_note_name(pitch / 100)
                self.kit_grid.SetCellValue(i * 2 + instr, instr_index+4, str(real_note))

    def load_kit(self, event):
        openFileDialog = wx.FileDialog(self, "Load Kit to current set", "", "",
                                       "HPD-20 Kit (*.kit)|*.kit",
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        openFileDialog.ShowModal()
        openFileDialog.GetPath()
        openFileDialog.Destroy()
        self.fill_kit(2)

    def save_kit(self, event):
        openFileDialog = wx.FileDialog(self, "Save Kit", "", "",
                                       "HPD-20 Kit (*.kit)|*.kit",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        openFileDialog.ShowModal()
        openFileDialog.GetPath()
        openFileDialog.Destroy()
        self.fill_kit(3)

    def load_memory_backup(self, event):
        openFileDialog = wx.FileDialog(self, "Open a memory dump", "", "",
                                       "HPD-20 Dump (*.HS0)|*.HS0",
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        openFileDialog.ShowModal()
        openFileDialog.GetPath()
        openFileDialog.Destroy()

    def save_memory_backup(self, event):
        saveFileDialog = wx.FileDialog(self, "Save memory dump as", "", "",
                                       "HPD-20 Dump (*.HS0)|*.HS0",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        saveFileDialog.ShowModal()
        saveFileDialog.GetPath()
        saveFileDialog.Destroy()

def run_main():
    app = wx.App(False)
    frame = MyForm('Backup/BKUP-022.HS0')
    frame.Show()
    app.MainLoop()

if __name__ == "__main__":
    run_main()




from os import mkdir

import sys

from instrumentname import get_instrument_name, get_instrument_pitch
from melodypadpattern import melody_pad_pattern
from pad import Pads
from kit import Kits
import hashlib

from scales import Scale

version = "0.0.4"


def get_note_name(value):
    notes = [' C', 'C#', ' D', 'Eb',
             ' E', ' F', 'F#', ' G',
             'Ab', ' A', 'Bb', ' B']
    if value < 0:
        return "--"
    return notes[value % len(notes)] + str(int((value - 2*len(notes)) / len(notes)))


class HPD:
    CHAIN_MEMINDEX = 1180
    CHAIN_MEMSIZE = 128

    KIT_MEMINDEX = 6922
    KIT_MEMSIZE = 224

    PAD_MEMINDEX = 51596
    PAD_MEMSIZE = 68

    CHAINS_COUNT = 15
    PADS_PER_KIT = 17
    KITS_COUNT = 200
    PADS_COUNT = KITS_COUNT * PADS_PER_KIT

    def __init__(self, file_name):
        fh = file(file_name, 'rb')
        memory_block = bytearray(fh.read())
        self.md5_memory = memory_block[-16:]
        print(" ".join(hex(n) for n in self.md5_memory))

        self.memoryBlock = memory_block[0:-16]   # strip md5
        pads_memory = memory_block[HPD.PAD_MEMINDEX:HPD.PAD_MEMINDEX + HPD.PADS_COUNT * HPD.PAD_MEMSIZE]
        kits_memory = memory_block[HPD.KIT_MEMINDEX:HPD.KIT_MEMINDEX + HPD.KITS_COUNT * HPD.KIT_MEMSIZE]
        self.kits = Kits(kits_memory)
        self.pads = Pads(pads_memory)

    def digest_kits(self):
        result = "Kits:\n"
        for i in range(HPD.KITS_COUNT):
            result += str(i+1) + ':\t'
            result += self.kits.get_kit(i).main_name().strip()
            result += '\t' + self.kits.get_kit(i).sub_name().strip()
            result += '\n'
        return result

    def digest_single_kit(self, kit):
        kit -= 1  # zero based

        result = "Kit {0}: {1} {2}\n".format(kit + 1, self.kits.get_kit(kit).main_name(), self.kits.get_kit(kit).sub_name())
        for i in range(0, HPD.PADS_PER_KIT):
            pitch = self.pads.get_pad(kit * HPD.PADS_PER_KIT + i).get_pitch(0)
            pitch = pitch + get_instrument_pitch(self.pads.get_pad(kit * HPD.PADS_PER_KIT + i).get_patch(0))
            real_note = get_note_name(pitch / 100)
            result += "{0}: {1} {2} {3} {4}\n".format(self.pads.get_pad_name(i), real_note,
                                                      self.pads.get_pad(kit * HPD.PADS_PER_KIT + i).get_volume(0),
                                                      get_instrument_name(self.pads.get_pad(kit * HPD.PADS_PER_KIT + i).get_patch(0)),
                                                      self.pads.get_pad(kit * HPD.PADS_PER_KIT + i).get_pitch(0))
        result += '\n'
        return result

    def save_file(self, file_name):
        m = hashlib.md5()
        m.update(str(self.memoryBlock))
        md5_digest = m.digest()
        fh = file(file_name, 'wb')
        fh.write(self.memoryBlock)
        print(" ".join(hex(ord(n)) for n in md5_digest))
        fh.write(md5_digest)

    def save_kit(self, kit_index, file_name, inc_filename_ifexists=False):
        try:
            if inc_filename_ifexists:
                pass

            fh = file(file_name, 'wb')
            kit = self.kits.get_kit(kit_index)
            kit.save(fh)
            for pad_index in range(HPD.PADS_PER_KIT):
                pad = self.pads.get_pad(HPD.PADS_PER_KIT*kit_index+pad_index)
                pad.save(fh)
        except Exception as e:
            print("Exception {0}".format(e))

    def load_kit(self, kit_index, file_name):
        try:
            fh = file(file_name, 'rb')
            kit = self.kits.get_kit(kit_index)
            kit.load(fh)
            for pad_index in range(HPD.PADS_PER_KIT):
                final_pad_index = HPD.PADS_PER_KIT*kit_index+pad_index
                pad = self.pads.get_pad(final_pad_index)
                pad.load(fh)
                self.memoryBlock[HPD.PAD_MEMINDEX + HPD.PAD_MEMSIZE * final_pad_index:HPD.PAD_MEMINDEX + HPD.PAD_MEMSIZE * (final_pad_index + 1)] = pad.memory_block
            self.memoryBlock[HPD.KIT_MEMINDEX + HPD.KIT_MEMSIZE * kit_index:HPD.KIT_MEMINDEX + HPD.KIT_MEMSIZE * (kit_index+1)] = kit.memory_block
        except Exception as e:
            print("Exception {0}".format(e))

    def create_kit_filename(self, kit_index):
        kit = self.kits.get_kit(kit_index)
        kit_name = kit.main_name().strip()
        if len(kit.sub_name().strip()) != 0:
            kit_name += " (" + kit.sub_name().strip() + ")"
        kit_as_file_name = "kits/" + "".join(
            [x if x.isalnum() or x in ['!', '&', '(', ')', '_', '.', '{', '}', ' '] else "_" for x in kit_name])
        return kit_as_file_name + ".kit"

    def save_all_kits(self):
        for kit_index in range(HPD.KITS_COUNT):
            self.save_kit(kit_index, self.create_kit_filename(kit_index))

    def apply_pad(self, pad_index):
        pad = self.pads.get_pad(pad_index)
        self.memoryBlock[HPD.PAD_MEMINDEX + HPD.PAD_MEMSIZE * pad_index:HPD.PAD_MEMINDEX + HPD.PAD_MEMSIZE * (
                        pad_index + 1)] = pad.memory_block

    def apply_scale(self, instrument_name, scale, mode, first_note, kit_index, pad_list):
        kit_index -= 1  # zero based
        instruments = Scale.get_scale(instrument_name, first_note, len(pad_list), scale, mode)

        for i in range(len(pad_list)):
            final_pad_index = HPD.PADS_PER_KIT * kit_index + pad_list[i]
            instrument_number = instruments[i][0]
            instrument_pitch = instruments[i][1]
            pad = self.pads.get_pad(final_pad_index)
            pad.set_patch(0, instrument_number)
            pad.set_pitch(0, instrument_pitch)
            self.apply_pad(final_pad_index)




def usage(argv):
    print("Usage: " + argv[0] + "[OPTIONS] <backup-file> [COMMAND]")
    print("Version: " + version)
    print("OPTIONS: ")
    print("  -h,--help              Show this help")
    print("  -v,--verbose           Create verbose output")

    print("COMMANDS:")
    print("  show chains")
    print("  show kits")
    print("  show kit #")

def run_main():
    argv = list()
    for arg in sys.argv:
        argv.append(arg)
    verbose = 0
    if len(argv) < 2:
        usage(argv)
        sys.exit(2)
    arg_pos = 1

    while argv[arg_pos].startswith('-'):
        if argv[arg_pos].startswith('--'):
            option = argv[arg_pos][2:]
        else:
            option = argv[arg_pos]
        arg_pos += 1
        if option == 'verbose' or option == '-v':
            verbose += 1
        elif option == 'help' or option == '-h':
            usage(argv)
            sys.exit(2)
        else:
            print("unknown option --{0}".format(option))
            usage(argv)
            sys.exit(2)

    hpd = HPD(argv[arg_pos])
    arg_pos += 1
    result = ""
    while arg_pos < len(argv):
        operation = argv[arg_pos]
        arg_pos += 1
        if operation == 'show':
            showing = argv[arg_pos]
            arg_pos += 1
            if showing == 'kits':
                result += hpd.digest_kits()
            if showing == 'kit':
                result += hpd.digest_single_kit(int(argv[arg_pos]))
                arg_pos += 1

    if result is not None:
        sys.stdout.write(result)
    sys.stdout.write('\n')

if __name__ == "__main__":
    run_main()

# some tests for now
'''
hpd = HPD('Backup/BKUP-022.HS0')

hpd.showkit(130)
hpd.showkit(151)
hpd.showkit(53)
hpd.showkit(130)
hpd.apply_scale("Glockenspiel", "major", Scale.IONIAN, 72+12, 130, melody_pad_pattern["Alternate-LR"])
hpd.showkit(130)
# hpd.load_kit(188, "kits/Brushes.kit")
hpd.save_file('BKUP-001.HS0')

# mkdir('kits')
# hpd.save_all_kits()
'''

